local tconcat, tinsert = table.concat, table.insert
local gmatch = string.gmatch

local AsmoRaidTracker = AsmoRaidTracker

local RARITIES = {
	[0]="Poor",
	[1]="Common",
	[2]="Uncommon",
	[3]="Rare",
	[4]="Epic",
	[5]="Legendary",
	[6]="Artifact",
	[7]="Heirloom",
}

local LOCALES = {
	["frFR"] = "Fançais",
	["enUS"] = "English",
}

local SYNCABLE_CONFIG = {
    itemRarityThreshold = true,
    language = true,
    excludedGuildRanks = true,
    officers = true,
    itemBlacklist = true,
    evalDkpValueString = true,
}

AsmoRaidTracker.options = {
    name = "AsmoRaidTracker",
    type = "group",
    args = {
        config = {
            type = "execute",
            name = "Configuration",
            desc = "Ouvre la configuration",
            func = function(info)
                InterfaceOptionsFrame_OpenToCategory(AsmoRaidTracker.optionsFrame)
			end,
			dialogHidden = true,
			order = 0
        },
        raid = {
            type = "execute",
            name = "Fenêtre de raid",
            desc = "Ouvre la fenêtre de gestion de raids",
            func = function(info)
            	local self = AsmoRaidTracker
            	if not self.displayState.isWindowOpened then
            		self:DrawWindow()
            	end
			end,
			order = 1,
        },
        invite = {
            type = "execute",
            name = "Gestion invites",
            desc = "Ouvre la fenêtre de gestion des invites",
            func = function(info)
            	local self = AsmoRaidTracker
            	if not self.inviter.displayState.isWindowOpened then
            		self.inviter:DrawWindow()
            	end
			end,
			order = 2,
        },
        debug = {
            type = "toggle",
            name = "Mode debug",
            desc = "Inverse le mode de debug",
            get = function(info)
            	return AsmoRaidTracker.db.profile.debug
			end,
			set = function(info, v)
				AsmoRaidTracker.db.profile.debug = v
			end,
			order = 3,
        },
        outputCommLog = {
            type = "execute",
            name = "Log messages",
            desc = "Affiche la log des 10 derniers messages",
            func = function(info)
            	AsmoRaidTracker:OutputCommLog()
			end,
			order = 4,
        },
        fullSynchro = {
            type = "input",
            name = "Full synchro - gaffe",
            desc = "Attention la full synchro est à ne pas utiliser à la légère",
            set = function(info, v)
            	AsmoRaidTracker.synchronizer:BroadcastFullSynchro(v)
			end,
			dialogHidden = true,
			order = 5,
        },
        synchronizedConfig = {
        	name = "Paramètres synchronisés",
        	type = "header",
		    order = 10,
		},
        officers = {
            type = "input",
            name = "Officiers",
            desc = "Officiers",
            get = function(info)
            	return tconcat(AsmoRaidTracker.db.profile.officers, ",")..","
            end,
            set = function(info, v)
            	local self = AsmoRaidTracker
            	self.db.profile.officers = {}
            	local tmpOfficers = {}
            	for officer in gmatch(v .. ",", "(.-),") do
            		if not tmpOfficers[officer] then
            			tmpOfficers[officer] = true
            			if officer ~= "" then
            				tinsert(self.db.profile.officers, officer)
            			end
            		end
            	end
            	self.synchronizer:SetOfficers(self.db.profile.officers)
            end,
		    order = 11,
        },
        itemBlacklist = {
            type = "input",
            name = "Liste noire items",
            desc = "Les items dont l'id est dans cette liste ne seront pas trackés",
            get = function(info)
            	local list = ""
            	for item, _ in pairs(AsmoRaidTracker.db.profile.itemBlacklist) do
            		list = list .. item .. ","
            	end
            	return list
            end,
            set = function(info, v)
            	local self = AsmoRaidTracker
            	self.db.profile.itemBlacklist = {}
            	local itemBlacklist = self.db.profile.itemBlacklist
            	for item in gmatch(v .. ",", "(.-),") do
            		if not itemBlacklist[item] then
            			if item ~= "" then
            				itemBlacklist[item+0]=true
            			end
            		end
            	end
            end,
		    order = 11,
        },
		itemRarity = {
            type = "select",
            name = "Rareté minimum",
            desc = "Rareté minimum pour qu'un loot soit pris en compte",
            values = RARITIES,
            get = function(info)
            	return AsmoRaidTracker.db.profile.itemRarityThreshold
            end,
            set = function(info, v)
            	AsmoRaidTracker.db.profile.itemRarityThreshold = v
            end,
		    order = 12,
        },
		evalDkpValue = {
            type = "input",
            multiline = true,
            name = "Evaluation valeur DKP",
            desc = "Entrez ici la formule de calcul du DKP d'un item, syntaxe lua standard, vous pouvez utiliser les variables itemLevel et itemLevel.",
            get = function(info)
            	return AsmoRaidTracker.db.profile.evalDkpValueString
            end,
            set = function(info, v)
            	AsmoRaidTracker.db.profile.evalDkpValueString = v
            end,
		    order = 13,
        },
        language = {
            type = "select",
            name = "Langue",
            desc = "Langue dans laquelle les items sont enregistrés",
            values = LOCALES,
            get = function(info)
            	return AsmoRaidTracker.db.profile.language
            end,
            set = function(info, v)
            	AsmoRaidTracker.db.profile.language = v
            end,
		    order = 14,
        },
        guildRanks = {
            type = "multiselect",
            name = "Rangs à exclure",
            desc = "Rangs de guilde à exclure pour l'ajout de membre à un raid",
            values = function()
            	local ranks = {}
            	for i=0, GuildControlGetNumRanks()-1 do
            		ranks[i]=GuildControlGetRankName(i+1)
            	end
            	return ranks
            end,
            get = function(info, k)
            	return AsmoRaidTracker.db.profile.excludedGuildRanks[k]
            end,
            set = function(info, k, v)
            	AsmoRaidTracker.db.profile.excludedGuildRanks[k] = v
            end,
		    order = 15,
        },
        synchronizeConfig = {
            type = "execute",
            name = "Synchroniser config",
            desc = "Synchroniser la configuration des utilisateurs en ligne",
            func = function(info)
                local self = AsmoRaidTracker
                local config = {}
                for k, v in pairs(self.db.profile) do
                    if SYNCABLE_CONFIG[k] then
                        config[k]=self.db.profile[k]
                    end
                end
                self.synchronizer:SendConfig(config)
			end,
		    order = 16,
        },
    },
}